#####################################################################
# VaR i ES dla symulacji historycznej                               #
# Zienne wejsciowe:                                                 #                       
# y     <- szereg czasowy                                           #
# p     <- poziom tolerancji                                        #
#####################################################################

VaRhist <- function(y, p){

  y1       <- sort(y)              
  N0       <- floor(length(y1)*p)       # obserwacja dla p-tego kwintyla                   
  VaR      <- y1[N0]                    # p-ty kwintyl
  y2       <- y1[1:N0]
  ES       <- mean(y2)                  # numeryczne calkowanie dla stop zwrotu
return( list(VaR = VaR, ES = ES) )
}

#####################################################################
# VaR i ES dla rozkladu normalnego                                  #
# Zienne wejsciowe:                                                 #                       
# y     <- szereg czasowy                                           #
# p     <- poziom tolerancji                                        #
#####################################################################

VaRnorm <- function(y, p){
  m     <- mean(y)
  s     <- sd(y)
  VaR   <- qnorm(p)*s + m            # VaR dla stopy zwrotu
  ES    <- m - s*dnorm(qnorm(p))/p   # Expected shortfall 
  return( list(VaR = VaR, ES = ES) )
}


#####################################################################
# VaR i ES dla rozkladu t-Studenta                                  #
# Zienne wejsciowe:                                                 #                       
# y     <- szereg czasowy                                           #
# v     <- liczba stopni swobody                                    #
# p     <- poziom tolerancji                                        #
#####################################################################

VaRt <- function(y, p, v=5){
  m      <- mean(y)
  s      <- sd(y)
  VaR    <- m + s*sqrt((v-2)/v)*qt(p,v)  
  qf     <- function(x) qt(x,v)
  ES     <- m + s*sqrt((v-2)/v)*(1/p * integrate(qf, 0, p)$value)   
  return(list(VaR = VaR, ES = ES)) 
}

#####################################################################
# VaR i ES z rozszerzenia Cornisha-Fischera                         #
# Zienne wejsciowe:                                                 #                       
# y     <- szereg czasowy                                           #
# p     <- poziom tolerancji                                        #
#####################################################################

VaRCF <- function(y, p){
  require(moments)
  m   = mean(y)
  s   = sd(y)
  S   = skewness(y)
  K   = kurtosis(y)
  PsiP <- qnorm(p)
  VaR  <- m + s*(PsiP + (PsiP^2-1)/6*S + (PsiP^3-3*PsiP)/24*(K-3) - (2*PsiP^3-5*PsiP)/36*(S^2) ) 
  return(list(VaR = VaR, ES = NaN)) 
}


#####################################################################
# VaR i ES z modelu EWMA (rozklad normalny lub t-Studenta           #
# Zienne wejsciowe:                                                 #                       
# y     <- szereg czasowy                                           #
# p     <- poziom tolerancji                                        #
# v     <- liczba stopni swobody dla rozkladu t-Studenta            #
#####################################################################

VaREWMA <- function(y, p, v=NULL, lambda=0.94){
  
  require(rugarch)
  
  # normalny czy t-Studenta
  if(is.null(v)){
      spec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=FALSE),
            variance.model=list(model="iGARCH", garchOrder=c(1,1)),
            fixed.pars=list(alpha1=1-lambda, omega=0), distribution.model = "norm")
    } else {
      spec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=FALSE),
                         variance.model=list(model="iGARCH", garchOrder=c(1,1)),
                         fixed.pars=list(alpha1=1-lambda, omega=0, shape=v), distribution.model = "std")
    }
  suppressWarnings(fit <- ugarchfit(data = y, spec = spec))
  
  s     <- sqrt(as.numeric(tail(y,1)^2) + as.numeric(tail(sigma(fit),1)^2))
  if(is.null(v)){
    VaR  <- s*qdist("norm", p) 
    qf   <- function(x) qdist("norm", p=x)
  } else {
    VaR  <- s*qdist("std", p, shape=v) 
    qf   <- function(x) qdist("std", p=x, shape=v)
  }
  ES     <- s*(1/p * integrate(qf, 0, p)$value)   
  return(list(VaR = VaR, ES = ES)) 
}

#####################################################################
# VaR i ES z modelu IGARCH (rozklad normalny lub t-Studenta         #
# Zienne wejsciowe:                                                 #                       
# y     <- szereg czasowy                                           #
# p     <- poziom tolerancji                                        #
# v     <- liczba stopni swobody dla rozkladu t-Studenta (>2)       #
#####################################################################

VaRIGARCH <- function(y, p, v=NULL, lambda=0.94){
  
  require(rugarch)
  # normalny czy t-Studenta
  if(is.null(v)){
    spec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=FALSE),
                       variance.model=list(model="iGARCH", garchOrder=c(1,1)),
                       fixed.pars=list(omega=0), start.pars = list(alpha1=1-lambda), distribution.model = "norm")
  } else {
    spec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=FALSE),
                       variance.model=list(model="iGARCH", garchOrder=c(1,1)),
                       fixed.pars=list(omega=0, shape=v), start.pars = list(alpha1=1-lambda), distribution.model = "std")
  }
  suppressWarnings(fit <- ugarchfit(data = y, spec = spec))
  suppressWarnings(fct   <- ugarchforecast(fit,n.ahead = 1))
  
  s     <- as.numeric(sigma(fct))
  if(is.null(v)){
    VaR  <- s*qdist("norm", p) 
    qf   <- function(x) qdist("norm", p=x)
  } else {
    VaR  <- s*qdist("std", p, shape=v) 
    qf   <- function(x) qdist("std", p=x, shape=v)
  }
  ES     <- s*(1/p * integrate(qf, 0, p)$value)   
  return(list(VaR = VaR, ES = ES)) 
}

#####################################################################
# VaR i ES z modelu GARCH (rozklad normalny lub t-Studenta          #
# Zienne wejsciowe:                                                 #                       
# y     <- szereg czasowy                                           #
# p     <- poziom tolerancji                                        #
# v     <- liczba stopni swobody dla rozkladu t-Studenta (>2)       #
# P,Q   <- GARCH lags                                               #
# eGARCH <- set TRUE for EGRACH model                               #
#####################################################################

VaRGARCH <- function(y, p, v=NULL, P=1, Q=1, eGARCH=FALSE){
  
  require(rugarch)
  
  mdl <- "sGARCH"
  if(eGARCH){mdl <- "eGARCH"}
  
  # normalny czy t-Studenta
  if(is.null(v)){
    spec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=TRUE),
                       variance.model=list(model=mdl, garchOrder=c(P,Q)), distribution.model = "norm")
  } else {
    spec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=TRUE),
                       variance.model=list(model=mdl, garchOrder=c(P,Q)), fixed.pars=list(shape=v), distribution.model = "std")
  }
  suppressWarnings(fit <- ugarchfit(data = y, spec = spec))
  suppressWarnings(fct <- ugarchforecast(fit,n.ahead = 1))
  m     <- as.numeric(fitted(fct))
  s     <- as.numeric(sigma(fct))
  
  if(is.null(v)){
    VaR  <- m + s*qdist("norm", p) 
    qf   <- function(x) qdist("norm", p=x)
  } else {
    VaR  <- m + s*qdist("std", p, shape=v) 
    qf   <- function(x) qdist("std", p=x, shape=v)
  }
  ES     <- m + s*(1/p * integrate(qf, 0, p)$value)   
  return(list(VaR = VaR, ES = ES)) 
}