rm(list=ls())
# Pakiet do zapisu danych w formacie zoo
require(zoo)
require(ggplot2)
require(rugarch)
require(MASS)
require(moments)

source("LoadFundData.R")
# source("LoadWigData.R")

# Panel wykresow: pelna proba 
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(P, main="Poziom", xlab="", ylab="")
plot(r, main="stopy zwrotu", xlab="", ylab="")

##########################################################
# Ustawienia do modelu VaR                               #
##########################################################
 
T  <- length(r)                          # liczba obserwacji
p  <- 0.05                               # poziom tolerancji VaR
H  <- 1                                  # horyont

N  <- 1250                               # ustalamy probe na podstawie ktorej liczymy VaR
r  <- tail(r,N)                          # stopy zwrotu
R  <- coredata(r)

##########################################################
# Symulacja historyczna                                  #
##########################################################

# wartosc zagrozona VaR
R0       <- sort(R)              
N0       <- floor(N*p)                  # obserwacja dla p-tego kwintyla                   
VaR_HS  <- R0[N0]                      # porownaj z: quantile(R,p)
# Oczekiwana strata ES
ES_HS  <- mean(R0[1:N0])               # srednia wartosc stop zwrotu ponizej VaR

# wykres na histogramie
hist(R, main="Symulacja historyczna", xlab="stopa zwrotu", ylab="l. obserwacji", nclass = 500, col="black")
abline(v=c(VaR_HS,ES_HS) ,lwd=2, col=c("blue","red"),lty=c(2,2))
legend("left", c("VaR dla sym. hist.","ES dla sym. hist."), lty=c(2,2), col=c("blue","red"), bty="n")

# Przedzial ufnosci dla VaR 
d1   <- density(R)
x    <- d1$x                              # punkty obserwacji
temp <- tail(which(x<VaR_HS),1)
fhat <- d1$y[temp]

seVaRHS <- sqrt(p*(1-p)/(N*fhat^2))                        # odchylenie standardowe dla kwintyla
CI      <- c(VaR_HS-1.96*seVaRHS, VaR_HS+1.96*seVaRHS)   # 95% przedzial ufnosci
CI

##########################################################
# Model parametryczny                                    #
##########################################################

# rozklad normalny
m   <- mean(R) 
s   <- sd(R)

VaR_N     <- qnorm(p)*s + m            # VaR dla stopy zwrotu
ES_N      <- m - s*dnorm(qnorm(p))/p   # Expected shortfall 

# wykres na density plot
d2  <- density(rnorm(100000, mean=m,sd=s))
plot(d2, main="Rozklad normalny", xlab="stopa zwrotu", ylab="f. gestosci", col="black")
abline(v=c(VaR_N,ES_N) ,lwd=2, col=c("blue","red"),lty=c(2,2))
legend("left", c("VaR dla rozkl. norm.","ES dla rozkl. norm."), lty=c(2,2), col=c("blue","red"), bty="n")

##########################################################
# Symulacje Monete Carlo dla rozkadu normalnego         #
##########################################################

M      <- 10000     # liczba losowan w symulacjach Monte Carlo
m      <- mean(R) 
s      <- sd(R)

draws0    <- rnorm(M, mean=m, sd=s)
draws1    <- sort(draws0)                 # uporzadkowane obserwacje
M0        <- floor(M*p)                   # obserwacja dla p-tego kwintyla                   
VaR_N0    <- draws1[M0]                   # porownaj z: quantile(draws1,p)

# Oczekiwana strata ES
ES_N0     <- mean(draws1[1:M0])           # srednia wartosc

# Porownanie uzyskanych wartosci
c(VaR_N,VaR_N0)
c(ES_N,ES_N0)

#########################
# Grube ogony           #
#########################

# Rozklad t-Studenta
# standaryzacja
m  <- mean(R)
s  <- sd(R)
R0 <- (R-m)/s

# estymacja liczby stopni swobody
K     <- kurtosis(R0)
v0    <- 4+6/(K-3)         # metoda momentow

dt    <- fitdistr(R0, "t", m = 0, start = list(s=sqrt((v0-2)/v0), df=v0), lower=c(0.001,3))
v1    <- dt$estimate[[2]]
# liczenie VaR
VaR_t <- m + s*qt(p,v1) * sqrt((v1-2)/v1)           
# Oczekiwana strata
qf       <- function(x) qt(x, v1)
ES_t     <- m + s*sqrt((v1-2)/v1)*(1/p * integrate(qf, 0, p)$value)   

# Uwaga. Alternatywa to skorzystanie z pakietu ugarch
# require(rugarch)
# temp  <- fitdist("std", R0)
# v1    <- temp$pars["shape"]
# VaR_t <- m + s*qdist("std",shape=v1)
# qf    <- function(x) qdist("std", p=x, shape=v1)
# ES_t  <- m + s*(1/p * integrate(qf, 0, p)$value) 

draws <- rt(100000,v1)*sqrt((v1-2)/v1)*s + m
d3    <- density(draws)
plot(d3, main="Rozklad t-Studenta", xlab="stopa zwrotu", ylab="f. gestosci", xlim=c(min(R),max(R)),col="black")
abline(v=c(VaR_t,ES_t) ,lwd=2, col=c("blue","red"),lty=c(2,2))
legend("left", c("VaR dla rozkl. t","ES dla rozkl. t"), lty=c(2,2), col=c("blue","red"), bty="n")

# Rozszerzenie Cornisha-Fischera    #
#####################################

require(moments)
m   = mean(R)
s   = sd(R)
S   = skewness(R)
K   = kurtosis(R)

PsiP    <- qnorm(p)
VaR_CF  <- m + s*(PsiP + (PsiP^2-1)/6*S + (PsiP^3-3*PsiP)/24*(K-3) - (2*PsiP^3-5*PsiP)/36*(S^2) ) 

# Porwnanie 

# HS / NORM / t-Student na jednym wykresie #

plot(d1, main="Funkcje gestosci", xlab="stopa zwrotu", ylab="f. gestosci", xlim=c(min(R),max(R)),col="black", lwd=2)
lines(d2,col="red", lwd=2)
lines(d3,col="green", lwd=2)
abline(v=c(VaR_HS2,VaR_N1,VaR_t) ,lwd=2, col=c("black", "red","green"),lty=c(2,2,2))
abline(v=c(ES_HS2,ES_N1,ES_t) ,lwd=2, col=c("black", "red","green"),lty=c(3,3,3))
legend("left", c("sym. historyczna","rozklad normalny","rozklad t"), lty=c(1,1,1), col=c("black", "red","green"), bty="n")


Tabela           <- matrix(c(VaR_HS,VaR_N,VaR_t,VaR_CF, ES_HS,ES_N,ES_t,NaN),4,2)
colnames(Tabela) <- c("VaR","ES")
rownames(Tabela) <- c("sym. hist.", "rozk. norm.", "rozklad t", "Cornish-Fischer")
round(-100*Tabela,2)

# Funkcje i wykres VaR dla rnych p  #
#######################################
source("MRFzR_FunkcjeBlok1.R")
TabelaP = matrix(NA,100,5)
for(p in seq(0.001,0.1,0.001)){
  TabelaP[p*1000,] <- c(p,VaRhist(R,p)$VaR, VaRnorm(R,p)$VaR, VaRt(R,p,v1)$VaR, VaRCF(R,p)$VaR)
}

plot(TabelaP[,1],TabelaP[,2], main="Poziom tolerancji a VaR", 
     xlab="poziom tolerancji (p)", ylab="VaR", 
     ylim=c(min(TabelaP[,2:5]),max(TabelaP[,2:5])), xlim=c(0,0.1),col="black", lwd=2, type="l")
lines(TabelaP[,1],TabelaP[,3],col="red", lwd=2, type="l")
lines(TabelaP[,1],TabelaP[,4],col="green", lwd=2, type="l")
lines(TabelaP[,1],TabelaP[,5],col="orange", lwd=2, type="l")
legend("right", c("sym. historyczna","rozklad normalny","rozklad t","Cornish-Fischer"), lty=c(1,1,1,1), col=c("black", "red","green","orange"), bty="n")
abline(v=c(0.05) ,lwd=2, lty=2)

