rm(list=ls())
# Pakiet do zapisu danych w formacie zoo
require(zoo)
require(ggplot2)
require(rugarch)

source("LoadFundData.R")
# source("LoadWigData.R")

# Panel wykresow: pelna proba 
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(P, main="Poziom", xlab="", ylab="")
plot(r, main="stopy zwrotu", xlab="", ylab="")

##########################################################
# Ustawienia do modelu VaR                               #
##########################################################

T  <- length(r)                          # liczba obserwacji
p  <- 0.05                               # poziom tolerancji VaR
H  <- 1                                  # horyont

N     <- 1250                             # ustalamy probe na podstawie ktorej liczymy VaR (5 lat)
r     <- tail(r,N)                        # dane
R     <- coredata(r)

############################
# wspolzaleznosci w czasie #
############################
require(forecast)

# brak autokorelacja zwrotow
Acf(R)
Pacf(R)

# ale jest grupowanie zmiennosci
Acf(R*R)
Pacf(R*R)

# oraz efekt dzwigni
Ccf(R*R, R)

########################################################################
# Grupowanie wariancji a VaR / ES                                      #
# Dla uproszczenia zakladamy rozklad t studenta o 5 stopniach swobody  #
########################################################################

require(rugarch)

# Ustawienia, czy obliczenia dla rozkadu normalnego czy t-Studenta z 5 stopniami swobody
q    <- qdist("std", p=p, shape=5)
qf   <- function(x) qdist("std", p=x, shape=5)
# UWAGA: w pakiecie rugarch automatyczne losowanie z rokladu o parametrach mu=0, sig=1. Porownaj z:
# q    <- qt(p,5)*sqrt((5-2)/5)
# q    <- qdist("norm",p)
# qf   <- function(x) qdist("norm", p=x)


# 1. Staly rozklad
# t-Student o 5 stopniach swobody / rozklad normalny
#######################################################

mu    <- mean(r)
sigma <- sd(r)

# VaR i ES (jako calka numeryczna)
VaR_const   <- mu + sigma*q
ES_const <- mu + sigma*(1/p * integrate(qf, 0, p)$value)   

# Wykresy
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(merge( r, mu + 2*sigma, mu - 2*sigma), plot.type="single", col=c(1,2,2),
     main="log-zwroty vs +-2sd", ylab="" )
plot(merge(r, VaR_const, ES_const), plot.type="single", col=c(1,2,3), main=paste(100*p,"% VaR i ES z modelu o stalych param.",  sep=""), ylab="", xlab="" )
legend("bottomright", c("VaR", "ES"), lty=1, col=2:3)

# 2. Parametry zmienne w czasie: model sredniej ruchomej (MA) 
# rozkad t-Studenta z 5 stopniami swobody / rozkad normalny
###############################################################

# szerokosc okna
w_length = 20             

# rolowana srednia i odchylenie standardowe
MAmean  <- rollapply(r, width=w_length, mean, by=1, align="right")
MAstd   <- rollapply(r, width=w_length, sd,   by=1, align="right")
# opznione o jeden okres, aby przedzial ufnosci na kolejny okres by oparty o dane do momentu t-1
MAmean <- lag(MAmean, -1)
MAstd  <- lag(MAstd,  -1)
# VaR i ES (rozklad t-Studenta o 5 stopniach swobody)
MAvar <- MAmean + MAstd*q 
MAes  <- MAmean + MAstd*(1/p * integrate(qf, 0, p)$value)

# wykresy
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(merge( r, MAmean + 2*MAstd, MAmean - 2*MAstd), plot.type="single", col=c(1,2,2),
     main="log-zwroty vs +-2sd", ylab="" )
plot(merge( r, MAvar, MAes), plot.type="single", col=c(1,2,3), main=paste(100*p,"% VaR i ES z modelu MA",  sep=""), ylab="" )
legend("bottomright", c("VaR", "ES"), lty=1, col=2:3)

# obliczenia VaR i ES dla ostatniego okresu
mT      <- mean(coredata(tail(r,w_length)))
sT      <- sd(coredata(tail(r,w_length)))
VaR_MA  <- mT + sT*q
ES_MA   <- mT + sT*(1/p * integrate(qf, 0, p)$value)

# 3. Kalibrowany EWMA (RiskMetrics) 
# rozkad t-Studenta z 5 stopniami swobody / rozkad normalny
###############################################################

lambda      <- 0.94                          # parametr wygladzajacy
EWMAsig2    <- rep(0,N)
EWMAsig2[1] <- var(R)                        # punkt startowy
for (t in 2:N){             
  EWMAsig2[t] = lambda * EWMAsig2[t-1]  + (1-lambda) * R[t-1]^2
}
EWMAstd      <- EWMAsig2^0.5
# VaR i ES 
EWMAvar <- EWMAstd*q 
EWMAes  <- EWMAstd*(1/p * integrate(qf, 0, p)$value)

# wykresy
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(merge( r, -2*EWMAstd, 2*EWMAstd), plot.type="single", col=c(1,2,2),
     main="log-zwroty vs EWMA +-2sd", ylab="" )
plot(merge( r, EWMAvar, EWMAes), plot.type="single", col=c(1,2,3), main=paste(100*p,"% VaR i ES z kalibrowanego modelu EWMA",  sep=""), ylab="" )
legend("bottomright", c("VaR", "ES"), lty=1, col=2:3)

# obliczenia VaR i ES dla ostatniego okresu
mT  <- 0
s2T <- lambda * EWMAsig2[N]  + (1-lambda) * R[N]^2
sT  <- sqrt(s2T)
VaR_EWMA = mT + sT*q
ES_EWMA  = mT + sT*(1/p * integrate(qf, 0, p)$value)

# EWMA w pakiecie ugarch - wprowadzenie do modeli GARCH
###########################################################

lambda = 0.94

EWMAspec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=FALSE),
                       variance.model=list(model="iGARCH", garchOrder=c(1,1)),
                       fixed.pars=list(alpha1=1-lambda, omega=0, shape=5), distribution.model = "std")
#                      fixed.pars=list(alpha1=1-lambda, omega=0),         distribution.model = "norm")

# przefiltrowanie danych
EWMAfit <- ugarchfit(data = r, spec = EWMAspec)
# nasze parametry
round(coef(EWMAfit),3)

# porownanie szeregow uzyskanych dwoma metodami
plot(merge(sigma(EWMAfit),EWMAstd), plot.type="single", col=c(1,2))
tail(merge(sigma(EWMAfit),EWMAstd))
plot(EWMAfit, which=1)

# 4. Estymacja parametru lambda w modelu EWMA: model IGARCH  
###########################################################

IGARCHspec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=FALSE),
                       variance.model=list(model="iGARCH", garchOrder=c(1,1)),
                       fixed.pars=list(omega=0,shape=5), start.pars = list(alpha1=0.06), distribution.model = "std")
#                      fixed.pars=list(omega=0), start.pars = list(alpha1=0.06), distribution.model = "norm")

IGARCHfit <- ugarchfit(data = r, spec = IGARCHspec, solver="nlminb")
round(coef(IGARCHfit),3)

# VaR i ES 
IGARCHvar <- sigma(IGARCHfit)*q 
IGARCHes  <- sigma(IGARCHfit)*(1/p * integrate(qf, 0, p)$value)

# wykresy
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(IGARCHfit, which=1)
plot(merge( r, IGARCHvar, IGARCHes), plot.type="single", col=c(1,2,3), main=paste(100*p,"% VaR i ES z estymowanego modelu EWMA (IGARCH)",  sep=""), ylab="" )
legend("bottomright", c("VaR", "ES"), lty=1, col=2:3)

# Obliczenia VaR i ES dla ostatniego okresu
IGARCHfct   <- ugarchforecast(IGARCHfit,n.ahead = 1)
mT  <- as.numeric(fitted(IGARCHfct))
sT  <- as.numeric(sigma(IGARCHfct))

VaR_IGARCH  = mT + sT*q
ES_IGARCH   = mT + sT*(1/p * integrate(qf, 0, p)$value)

# 5. Estymacja wszystkich parametrow: GARCH(1,1)  
###########################################################

GARCHspec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=TRUE),
                         variance.model=list(model="sGARCH", garchOrder=c(1,1)),
                         fixed.pars=list(shape=5), distribution.model = "std")
#                        distribution.model = "norm")

GARCHfit <- ugarchfit(data = r, spec = GARCHspec, solver="nlminb")
round(coef(GARCHfit),5)

# VaR i ES (rozklad t-Studenta o 5 stopniach swobody)
GARCHvar   <- fitted(GARCHfit) + sigma(GARCHfit)*q 
GARCHes    <- fitted(GARCHfit) + sigma(GARCHfit)*(1/p * integrate(qf, 0, p)$value)

# wykresy
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(GARCHfit, which=1) 
plot(merge( r, GARCHvar, GARCHes), plot.type="single", col=c(1,2,3), main=paste(100*p,"% VaR i ES z modelu GARCH(1,1))",  sep=""), ylab="" )
legend("bottomright", c("VaR", "ES"), lty=1, col=2:3)

# Obliczenia VaR i ES dla ostatniego okresu
GARCHfct   <- ugarchforecast(GARCHfit,n.ahead = 1)
mT  <- as.numeric(fitted(GARCHfct))
sT  <- as.numeric(sigma(GARCHfct))

VaR_GARCH  = mT + sT*q
ES_GARCH   = mT + sT*(1/p * integrate(qf, 0, p)$value)

# 6. Efekt dwigni: model eGARCH(1,1)  
###########################################################

EGARCHspec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=TRUE),
                        variance.model=list(model="eGARCH", garchOrder=c(1,1)),
                        fixed.pars=list(shape=5), distribution.model = "std")
#                        distribution.model = "norm")

EGARCHfit <- ugarchfit(data = r, spec = EGARCHspec, solver="nlminb")
round(coef(EGARCHfit),5)

# VaR i ES (rozklad t-Studenta o 5 stopniach swobody)
EGARCHvar   <- fitted(EGARCHfit) + sigma(EGARCHfit)*q 
EGARCHes    <- fitted(EGARCHfit) + sigma(EGARCHfit)*(1/p * integrate(qf, 0, p)$value)

# wykresy
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(EGARCHfit, which=1)
plot(merge( r, EGARCHvar, EGARCHes), plot.type="single", col=c(1,2,3), main=paste(100*p,"% VaR i ES z modelu EGARCH(1,1))",  sep=""), ylab="" )
legend("bottomright", c("VaR", "ES"), lty=1, col=2:3)

# Obliczenia VaR i ES dla ostatniego okresu
EGARCHfct   <- ugarchforecast(EGARCHfit,n.ahead = 1)
mT  <- as.numeric(fitted(EGARCHfct))
sT  <- as.numeric(sigma(EGARCHfct))

VaR_EGARCH  = mT + sT*q
ES_EGARCH   = mT + sT*(1/p * integrate(qf, 0, p)$value)

# Wykresy dla zmiennosci sigma
##################################

windows()
labssd <- c("log-zwroty", "staly rozklad", "MA", "EWMA", "IGARCH", "GARCH", "eGARCH")
plot( merge(r,
            2*sigma,
            2*MAstd,
            2*EWMAstd,
            2*sigma(IGARCHfit),
            2*sigma(GARCHfit),
            2*sigma(EGARCHfit),
            -2*sigma,
            -2*MAstd,
            -2*EWMAstd,
            -2*sigma(IGARCHfit),
            -2*sigma(GARCHfit),
            -2*sigma(EGARCHfit)),
         facets=NULL,
         main="log-zwroty vs +-2sd",
     col= c(1:7, 2:7),
     plot.type="single",
     ylab="", xlab="")
legend("bottomright", labssd, lty=1, col=c(1:7, 2:7))


# Wykresy dla VaR 
###################
windows()
labsvar <- c("log-zwroty", "staly rozklad", "MA", "EWMA", "IGARCH", "GARCH", "eGARCH")
plot(merge(r,
           VaR_const,
           MAvar,
           EWMAvar,
           IGARCHvar,
           GARCHvar,
           EGARCHvar),
     main=paste("log-zwroty vs ", 100*p,"% VaR", sep=""),
     col=1:7,
     plot.type="single",
     ylab="", xlab="")
legend("bottomright", labsvar, lty=1, col=1:7)

# wszystko razem
-100*c(VaR_const,VaR_MA, VaR_EWMA, VaR_IGARCH, VaR_GARCH, VaR_EGARCH)
-100*c(ES_const,ES_MA, ES_EWMA, ES_IGARCH, ES_GARCH, ES_EGARCH)

# z wykorzystaniem funkcji w pliku MRFzR_FunkcjeBlok1.R
source("MRFzR_FunkcjeBlok1.R")
VaR_const1  <- VaRt(R,p,5)$VaR
VaR_MA1     <- VaRt(tail(R,w_length),p,5)$VaR
VaR_EWMA1   <- VaREWMA(r, p, v=5)$VaR  
VaR_IGARCH1 <- VaRIGARCH(r, p, v=5)$VaR
VaR_GARCH1  <- VaRGARCH(r, p, v=5)$VaR
VaR_EGARCH1 <- VaRGARCH(r, p, v=5, eGARCH=TRUE)$VaR


# Porownanie
-100*cbind(
c(VaR_const,VaR_MA, VaR_EWMA, VaR_IGARCH, VaR_GARCH, VaR_EGARCH),
c(VaR_const1,VaR_MA1, VaR_EWMA1, VaR_IGARCH1, VaR_GARCH1, VaR_EGARCH1))

