rm(list=ls())
# Pakiet do zapisu danych w formacie zoo
require(zoo)
require(ggplot2)
require(rugarch)

source("LoadFundData.R")
#source("LoadWigData.R")

# Panel wykresow: pelna proba 
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(P, main="Poziom", xlab="", ylab="")
plot(r, main="stopy zwrotu", xlab="", ylab="")

##########################################################
# Ustawienia do modelu VaR                               #
##########################################################
 
T  <- length(r)                          # liczba obserwacji
p  <- 0.05                               # poziom tolerancji VaR
H  <- 5                                  # horyzont

N     <- 1250                            # ustalamy probe na podstawie ktorej liczymy VaR
r     <- tail(r,N)                       # dane
R     <- coredata(r)

##########################################################
# Metoda square root of time dla rozkadu normalnego     #
##########################################################

m   <- mean(R) # m=0
s   <- sd(R)
mH  <- m*H
sH  <- s*sqrt(H)

VaR1_N    <- qnorm(p)*s + m            # VaR dla horyontu 1
ES1_N     <- m - s*dnorm(qnorm(p))/p   # Expected shortfall dla horyontu 1

VaRH_N    <- mH + sH*qnorm(p)            # VaR dla stopy zwrotu dla horyontu H
ESH_N     <- mH - sH*dnorm(qnorm(p))/p   # Expected shortfall dla horyontu H


# wykres na density plot
x = seq(-4*sH,4*sH,0.01*sH)
df1 <- as.data.frame(cbind(x,dnorm(x,m,s))); names(df1) <- c("r","y"); df1$horyzont = "h=1" 
dfH <- as.data.frame(cbind(x,dnorm(x,mH,sH))); names(dfH) <- c("r","y"); dfH$horyzont = "h=H" 
df  = rbind(df1,dfH)

ggplot(data = df, aes(x = r, y=y, color=horyzont)) + 
  geom_line(size=1.2) + 
  scale_color_grey() + 
  theme_classic() +
  labs(title="VaR dla dalszych horyzontow", y="", x="st. zwrotu", caption=" ")+
  geom_vline(xintercept=c(VaRH_N,VaR1_N), linetype="dashed", size=1.2, color=c("gray","black"))  

#####################################
# Rozszerzenie Cornisha-Fischera    #
#####################################
require(moments)
m   = mean(R)
s   = sd(R)
S   = skewness(R)
K   = kurtosis(R)-3

PsiP    <- qnorm(p)
VaR1_CF  <- m + s*(PsiP + (PsiP^2-1)/6*S + (PsiP^3-3*PsiP)/24*(K) - (2*PsiP^3-5*PsiP)/36*(S^2) ) 

mH  <- m*H
sH  <- s*sqrt(H)
SH  <- S/sqrt(H)
KH  <- K/H

VaRH_CF        <- mH + sH*(PsiP + (PsiP^2-1)/6*SH + (PsiP^3-3*PsiP)/24*(KH) - (2*PsiP^3-5*PsiP)/36*(SH^2) ) 
# alternatywny wzor
# VaRHCF        <- m*H + s*sqrt(H)*(PsiP + (PsiP^2-1)/6*(S/sqrt(H)) + (PsiP^3-3*PsiP)/24*(K/H) - (2*PsiP^3-5*PsiP)/36*(S^2/H) ) 

round(-100*c(VaR1_N, VaR1_CF),2)
round(-100*c(VaRH_N, VaRH_CF),2)

##########################################################
# Symulacje Monete Carlo dla rozkladu normalnego         #
##########################################################

# liczba losowan w symulacjach Monte Carlo
M      <- 10000      

# macierz do ktorej zapisujemy losowania sciezek stop zwrotu
draws  <- matrix(NA,H,M)
# uzupelniamy kolejne kolumny macierzy
for(i in 1:M){
  draws[,i] <- rnorm(H, mean=m, sd=s)
}

# metoda szybsza numerycznie
# draws <- matrix(rnorm(M*H, mean=m, sd=s),nrow=H,ncol=M)

draws0      <- colSums(draws)               # skumulowane zwroty
draws0      <- sort(draws0)                 # uporzadkowane obserwacje
M0          <- floor(M*p)                   # obserwacja dla p-tego kwintyla                   
VaRH_NMC    <- draws0[M0]                   # porownaj z: quantile(draws0,p)

# Oczekiwana strata ES
ESH_NMC     <- mean(draws0[1:M0])           # numeryczne calkowanie 

# Porownanie uzyskanych wartosci
round(-100*c(VaRH_N, VaRH_NMC),2)
round(-100*c(ESH_N, ESH_NMC),2)

##########################################################
# Symulacje Monte Carlo dla rozkladu t-Studenta          #
##########################################################
require(MASS)
# liczba losowan w symulacjach Monte Carlo
M      <- 10000       
# Estymacja parametrow
R0 <- (R-m)/s
# Liczba stopni swobody
v     <- 5                                  # wartosc poczatkowa
dt    <- fitdistr(R0, "t", m = 0, start = list(s=sqrt((v-2)/v), df=v), lower=c(0.001,3))
v     <- dt$estimate[[2]]                   # wartosc oszacowana

# Symulacje monte carlo 
draws  <- matrix(NA,H,M)

# uzupelniamy kolejne kolumny macierzy
for(i in 1:M){
  draws[,i] <- m + sqrt((v-2)/v)*rt(H, v)*s
}
draws1      <- colSums(draws) 
draws1      <- sort(draws1)                 # uporzadkowane obserwacje
M0          <- floor(M*p)                   # obserwacja dla p-tego kwintyla                   
VaRH_t      <- draws1[M0]                   # porownaj z: quantile(draws0,p)

# Oczekiwana strata ES
ESH_t       <- mean(draws1[1:M0])           # numeryczne calkowanie dla stop zwrotu

# Wykres 
df0 = as.data.frame(draws0); names(df0) = "draws"; df0$rozklad = "Normalny"
df1 = as.data.frame(draws1); names(df1) = "draws"; df1$rozklad = "t-Student"
df  = rbind(df0,df1)

ggplot(df, aes(x=draws, color=rozklad)) +
  geom_density(size=1.2)+
  scale_color_grey() + 
  theme_classic() +
  labs(title="Rozklad stop zwrotu dla dalszych horyzontow", y="", x="st. zwrotu", caption=" ") +
  geom_vline(xintercept=c(VaRH_t,VaRH_N), linetype="dashed", size=1.2, color=c("gray","black"))  

##########################################################
# Symulacje Monte Carlo dla modelu GARCH                 #
##########################################################
require(rugarch)

# liczba losowan w symulacjach Monte Carlo
M      <- 10000       

# Specyfikacja modelu: GARCH(1,1) z r. normalnym
garch.spec <- ugarchspec(variance.model = list(model="sGARCH", garchOrder=c(1,1)),
                        mean.model = list(armaOrder=c(0,0), include.mean=TRUE),
                        distribution.model = "norm")
#estymacja modelu
garch.fit  <- ugarchfit(data=R, spec=garch.spec)

# symulacje MC z modelu GARCH
garch.sim   <- ugarchsim(garch.fit, n.sim = H, n.start = 0, m.sim = M, startMethod = "sample")
draws       <- garch.sim@simulation$seriesSim

# Obliczenia VaR
draws2      <- colSums(draws) 
draws2      <- sort(draws2)                 # uporzadkowane obserwacje
M0          <- floor(M*p)                   # obserwacja dla p-tego kwintyla                   
VaRH_GARCH  <- draws2[M0]                   # porownaj z: quantile(draws0,p)

# Oczekiwana strata ES
ESH_GARCH   <- mean(draws2[1:M0])           # numeryczne calkowanie dla stop zwrotu

# wykres
df2 = as.data.frame(draws2); names(df2) = "draws"; df2$rozklad = "GARCH"
df  = rbind(df0,df2)

ggplot(df, aes(x=draws, color=rozklad)) +
  geom_density(size=1.2)+
  scale_color_grey() + 
  theme_classic() +
  labs(title="Rozklad stop zwrotu dla dalszych horyzontow", y="", x="st. zwrotu", caption=" ") +
  geom_vline(xintercept=c(VaRH_N,VaRH_GARCH), linetype="dashed", size=1.2, color=c("gray","black"))  


##########################################################
# BOOTSTRAP - symulacja historyczna                      #
##########################################################

require(MASS)
# liczba powtorzen bootrapowych
M      <- 10000       

# Symulacje monte carlo 
draws  <- matrix(NA,H,M)

# uzupelniamy kolejne kolumny macierzy
for(i in 1:M){
  draws[,i] <- sample(R, H, replace=TRUE)  # losujemy z historycznych stop zwrotu
}
draws3      <- colSums(draws) 
draws3      <- sort(draws3)                 # uporzadkowane obserwacje
M0          <- floor(M*p)                   # obserwacja dla p-tego kwintyla                   
VaRH_boot   <- draws3[M0]                   

# Oczekiwana strata ES
ESH_boot   <- mean(draws3[1:M0])           # numeryczne calkowanie dla stop zwrotu

# Wykres 
df3 = as.data.frame(draws3); names(df3) = "draws"; df3$rozklad = "Bootstrap"
df  = rbind(df0,df3)

ggplot(df, aes(x=draws, color=rozklad)) +
  geom_density(size=1.2)+
  scale_color_grey() + 
  theme_classic() +
  labs(title="Rozklad stop zwrotu dla dalszych horyzontow", y="", x="st. zwrotu", caption=" ") +
  geom_vline(xintercept=c(VaRH_N,VaRH_boot), linetype="dashed", size=1.2, color=c("gray","black"))  


########################################################
# A co jezeli stopy nie sa IID                          #
#########################################################

# Liczymy stopy H-okresowe (ele jest ich H-razy mniej...)

PH    <- tail(P,N)                # Wartosc WIG dla N ostatnich sesji
PH    <- PH[seq(N,1,-H)]          # obserwacje co H sesji
rH    <- diff(log(PH))            # H-okresowe stopy zwrotu
length(rH)

mNH   <- mean(rH) 
sNH   <- sd(rH)

VaRH_NH    <- qnorm(p)*sNH + mNH            # VaR dla horyontu H
ESH_NH     <- mNH - sNH*dnorm(qnorm(p))/p   # Expected shortfall dla horyontu H

round(-100*c(VaRH_NH,VaRH_N),3)
round(-100*c(ESH_NH,ESH_N),3)

x = seq(-4*sH,4*sH,0.01*sH)
dfNH <- as.data.frame(cbind(x,dnorm(x,mNH,sNH))); names(dfNH) <- c("r","y"); dfNH$method = "Stopy H-okresowe" 
dfH  <- as.data.frame(cbind(x,dnorm(x,mH,sH))); names(dfH) <- c("r","y"); dfH$method = "Stopy 1-okresowe" 
df  = rbind(dfNH,dfH)

ggplot(data = df, aes(x = r, y=y, color=method)) + 
  geom_line(size=1.2) + 
  scale_color_grey() + 
  theme_classic() +
  labs(title="VaR dla dalszych horyzontow", y="", x="st. zwrotu", caption=" ")+
  geom_vline(xintercept=c(VaRH_NH,VaRH_N), linetype="dashed", size=1.2, color=c("gray","black")) 
